library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ADC is
  port
  (
    clk    : in std_logic;
    SCLK   : out std_logic;        --ADC SCLK input
    CS     : out std_logic := '0'; --ADC CS input
    DOUT   : in std_logic;         --ADC DOUT output
    write1 : out std_logic;        -- start write to array
	 data   : out unsigned(11 downto 0)    
  );
end ADC;

architecture Behavioral of ADC is
  signal clk_z4 : unsigned(1 downto 0) := "00"; --set ADC clock
  signal clk_adc : std_logic := '0'; --clock to ADC
  signal CS_cnt : unsigned(3 downto 0) := "0000"; --set CS interval
  signal CS_sig : std_logic := '0'; --CS to ADC
  signal D_analog_in : unsigned(15 downto 0) := to_unsigned(0, 16); --temp. read
  signal i_tab : unsigned (3 downto 0) := "1101"; --write to temp. read

begin
  process (clk)
  begin
    if (rising_edge(clk)) then
      --clk_z4 <= not clk_z4;  AT, stavek brez pomena
      if clk_z4 >= "01" then --set ADC clock (ADC MAX11105AUT+T)
        clk_adc <= not clk_adc;
        clk_z4 <= "00";
      else
        clk_z4 <= clk_z4 + 1;
      end if;
      if clk_z4 = "01" and clk_adc = '0' then --rising_edge(clk_adc)
        i_tab <= i_tab - 1;
        CS_cnt <= CS_cnt + 1;
      end if;
      if CS_cnt = "1111" then --set ADC's "CS" input interval
        CS_sig <= '0';
      end if;
      if CS_cnt = "1101" then
        CS_sig <= '1';
      end if;
      if CS_cnt = "1110" and clk_z4 = "01" and clk_adc = '0' then --write before new read (write enable)
        write1 <= '1';
      else
        write1 <= '0';
      end if;

      if CS_cnt = "1111" then --falling_edge(CS_sig) --start write in temp. variable --write in array --set/reset "start_hold"
        if CS_sig = '1' then
          i_tab <= "1101";
			 data <= D_analog_in(11 downto 0); -- memory write data
        end if;

      end if;
      if clk_z4 = "01" and clk_adc = '1' then --falling_edge(clk_adc)
        if CS_sig = '0' then
          D_analog_in(to_integer(i_tab)) <= DOUT; --write into temp. variable for writing in array
        end if;
      end if;
      
    end if;
  end process;
  
  SCLK <= clk_adc;
  CS <= CS_sig;
end Behavioral;