library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity Master is
  port (
    clk : in std_logic;
    SCLK : out std_logic;							--ADC SCLK input
    CS : out std_logic := '0';							--ADC CS input
    DOUT : in std_logic;							--ADC DOUT output
    led : out std_logic_vector(7 downto 0);
    
	 SCLK_DAC : out std_logic;
	 SYNC : out std_logic;
	 DIN : out std_logic := '0'
  );
end Master;

architecture RTL of Master is

  signal start :  std_logic;							--start write to array
  signal read1 :  std_logic;							--enable reading next from array
  signal write1 :  std_logic := '0';	
  signal data :  unsigned(11 downto 0);
  signal adr : unsigned(6 downto 0);
  signal done :  std_logic;
  	
  signal source : std_logic_vector(19 downto 0);
  signal probe : std_logic_vector(11 downto 0);

  signal delta : std_logic_vector(9 downto 0);
  signal amplitude : std_logic_vector(4 downto 0);
  signal signal_select : std_logic_vector(2 downto 0);
  
  signal DAC_out : unsigned(9 downto 0) := to_unsigned(0, 10);
  
	component vmesnik is
		port (
			source : out std_logic_vector(19 downto 0);                    -- source
			probe  : in  std_logic_vector(11 downto 0)  := (others => 'X')  -- probe
		);
	end component vmesnik;
	
  
  component ADC is
	  port (
		 clk    : in std_logic;
		 SCLK   : out std_logic;        --ADC SCLK input
		 CS     : out std_logic := '0'; --ADC CS input
		 DOUT   : in std_logic;         --ADC DOUT output
		 write1 : out std_logic;        -- start write to array
		 data   : out unsigned(11 downto 0)    
	  );
  end component;
  
  component RAMctrl is
    port(
      clk   : in std_logic;
		start : in std_logic; -- start write to array
		write1: in std_logic; -- write new data
		din   : in unsigned(11 downto 0); -- input data
		
		read1 : in std_logic; --enable reading next from array
      done  : out std_logic := '0';	    --whole array read      
		dout  : out unsigned(11 downto 0);
	   adr   : out unsigned (6 downto 0)
    );
  end component; 
  
  
  component DAC is
    port(
	 	clk : in std_logic;
		
	   SCLK_DAC : out std_logic;
	   SYNC : out std_logic;
	   DIN : out std_logic;
		
      DAC_out : in unsigned(9 downto 0) := to_unsigned(0, 10)
	 );
  end component;
  
  
  component signal_gen is
    port(
	   clk : in std_logic;
      delta : in std_logic_vector(9 downto 0);
	   amplitude : in std_logic_vector(4 downto 0);
      signal_select : in std_logic_vector(2 downto 0);
	 
	   DAC_out : out unsigned(9 downto 0) := to_unsigned(0, 10)
	 );
  end component;
  
begin

u0 : component vmesnik
		port map (
			source => source, -- sources.source
			probe  => probe   --  probes.probe
		);
		

u1 : ADC port map(
  clk => clk,
  SCLK => SCLK,
  CS => CS,
  DOUT => DOUT,
  write1 => write1,
  data => data
);

u2 : RAMctrl port map(
  clk => clk,
  start => start,
  write1 => write1,
  din => data,
  read1 => read1,
  done => done,  
  dout => open,
  adr => adr
);


u3 : DAC port map(
  clk => clk,
  
  SCLK_DAC => SCLK_DAC,
  SYNC => SYNC,
  DIN => DIN,
  DAC_out => DAC_out
);


u4 : signal_gen port map(
  clk => clk,
  delta => delta,
  amplitude => amplitude,
  signal_select => signal_select,
	
  DAC_out => DAC_out
);

	
  probe <= std_logic_vector(data);
  start <= source(0);
  read1 <= source(1);
  led(0) <= done;
  led(2 downto 1) <= source(1 downto 0); 
  led(7 downto 3) <= std_logic_vector(adr(4 downto 0));
  delta <= source(14 downto 5);
  signal_select <= source(4 downto 2);
  amplitude <= source(19 downto 15);
  

end RTL;

	

	
